@@ -8,14 +8,15 @@ from zeam.form.base.errors import Error
 from zeam.form.base.markers import NO_VALUE, NOTHING_DONE, FAILURE
 from zeam.form.base.markers import getValue, DEFAULT
 
-from zope.interface import implements, alsoProvides
+from zope.publisher.interfaces.http import MethodNotAllowed
+from zope.interface import implementer, implements, alsoProvides
 from zope import component
 
 
+@implementer(interfaces.IAction)
 class Action(Component):
     """A form action.
     """
-    implements(interfaces.IAction)
 
     prefix = 'action'
     # By default an action is always in input mode (there is not much
@@ -23,7 +24,14 @@ class Action(Component):
     mode = 'input'
     description = None
     accesskey = None
-    postOnly = DEFAULT
+    html5Validation = True
+    methods = None
+    htmlAttributes = {}
+
+    def __init__(self, title=None, identifier=None, **htmlAttributes):
+        super(Action, self).__init__(title, identifier)
+        self.htmlAttributes = self.htmlAttributes.copy()
+        self.htmlAttributes.update(htmlAttributes)
 
     def available(self, form):
         return True
@@ -35,10 +43,10 @@ class Action(Component):
         raise NotImplementedError
 
 
+@implementer(interfaces.IActions)
 class Actions(Collection):
     """A list of form action.
     """
-    implements(interfaces.IActions)
 
     type = interfaces.IAction
 
@@ -49,25 +57,22 @@ class Actions(Collection):
 
             value, error = extractor.extract()
             if value is not NO_VALUE:
-                isPostOnly = getValue(action, 'postOnly', form)
-                if isPostOnly and request.method != 'POST':
-                    form.errors.append(
-                        Error('This form was not submitted properly',
-                              form.prefix))
-                    return form, None, FAILURE
+                methods = action.methods or form.methods
+                if methods and request.method.upper() not in methods:
+                    raise MethodNotAllowed(form.context, request)
                 try:
                     if action.validate(form):
                         return form, action, action(form)
-                except interfaces.ActionError, error:
+                except interfaces.ActionError as error:
                     form.errors.append(Error(error.args[0], form.prefix))
                     return form, action, FAILURE
         return form, None, NOTHING_DONE
 
 
-class CompoundActions(object):
+@implementer(interfaces.IIterable)
+class CompoundActions:
     """Compound different types of actions together.
     """
-    implements(interfaces.IIterable)
 
     def __init__(self, *new_actions):
         self.__actions = []
@@ -110,8 +115,9 @@ class DecoratedAction(Action):
 
     def __init__(self, title, callback,
                  identifier=None, description=None, accesskey=None,
-                 validator=None, available=None):
-        super(Action, self).__init__(title, identifier)
+                 validator=None, available=None, **htmlAttributes):
+        super(DecoratedAction, self).__init__(
+            title, identifier, **htmlAttributes)
         self._callback = callback
         self._validator = validator
         self._available = available
@@ -147,11 +153,11 @@ class ExtractedDecoratedAction(DecoratedAction):
 
 def action(title, identifier=None, description=None, accesskey=None,
            validator=None, available=None, implements=None,
-           factory=DecoratedAction, category='actions'):
+           factory=DecoratedAction, category='actions', **htmlAttributes):
     def createAction(callback):
         new_action = factory(
             title, callback, identifier, description, accesskey,
-            validator, available)
+            validator, available, **htmlAttributes)
         if implements is not None:
             alsoProvides(new_action, implements)
 
