@@ -1,12 +1,11 @@
-
 from zeam.form.base.interfaces import IDataManager
-from zope.interface import implements
+from zope.interface import implements, implementer
 
 
-class BaseDataManager(object):
+@implementer(IDataManager)
+class BaseDataManager:
     """Base class for a data manager.
     """
-    implements(IDataManager)
 
     def __init__(self, content):
         self.content = content
