@@ -1,9 +1,22 @@
-# -*- coding: utf-8 -*-
-
 from zope import interface
 from zope.publisher.interfaces.browser import IBrowserPage
 
 
+class IInvalidCSRFToken(interface.Interface):
+
+    def doc():
+        """The form submit could not be handled as the CSRF token is missing
+        or incorrect.
+        """
+
+
+@interface.implementer(IInvalidCSRFToken)
+class InvalidCSRFToken(Exception):
+    """The form submit could not be handled as the CSRF token is missing
+    or incorrect.
+    """
+
+
 class IModeMarker(interface.Interface):
     """This interface identifies a form mode and defines if it allows
     data extraction.
@@ -227,6 +240,9 @@ class IAction(IRenderableComponent):
     """A form action.
     """
     description = interface.Attribute(u"Describe the action")
+    accesskey = interface.Attribute(u"Accesskey for the action")
+    html5Validation = interface.Attribute(
+        u"Enable HTML5 validation for this action")
 
     def validate(form):
         """Self validation of values in order to run.
@@ -249,11 +265,16 @@ class IActions(ICollection):
 class IField(IRenderableComponent, IFieldExtractionValueSetting):
     """A form field.
     """
-    description = interface.Attribute(u"Field description")
+    description = interface.Attribute(
+        u"Field description")
     required = interface.Attribute(
         u"Boolean indicating if the field is required")
     readonly = interface.Attribute(
         u"Boolean indicating if the field is read-only")
+    htmlAttributes = interface.Attribute(
+        u"Dictionnary with extra html attributes to add to the field")
+    interface = interface.Attribute(
+        u"Optional Zope interface associated to the field")
 
     def getDefaultValue(form):
         """Return the default value.
@@ -294,6 +315,12 @@ class IErrors(IMutableCollection):
 class IWidget(IComponent):
     """Display a form component on the page.
     """
+    defaultHtmlClass = interface.Attribute(
+        u"List of default html class to apply to the widget")
+    defaultHtmlAttributes = interface.Attribute(
+        u"Set of default authorized html attributes (without data- attributes)")
+    alternateLayout = interface.Attribute(
+        u"Boolean indicating if the widget can be rendered in a compact way")
 
     def htmlId():
         """Return the HTML id of the HTML component representing the
@@ -304,6 +331,14 @@ class IWidget(IComponent):
         """Return an HTML class to mark the widget with.
         """
 
+    def htmlAttribute(name):
+        """Return the value of the given extra HTML attribute.
+        """
+
+    def htmlAttributes():
+        """Return a dictionary with all authorized extra HTML attributes.
+        """
+
     def isVisible():
         """Return True if the widget will render something visible in
         the rendered HTML.
@@ -338,11 +373,12 @@ class IWidgetFactory(interface.Interface):
 class IFieldWidget(IWidget):
     """Widget for fields.
     """
-    description = interface.Attribute(u"Description of the field")
-    error = interface.Attribute(u"Field error, or None")
+    description = interface.Attribute(
+        u"Description of the field")
+    error = interface.Attribute(
+        u"Field error, or None")
     required = interface.Attribute(
         u"Boolean indicating if the field is required")
-    readonly = interface.Attribute(u"Boolean indicating if field is read-only")
 
 
 class IWidgetExtractor(interface.Interface):
