@@ -1,12 +1,11 @@
-
 import unittest
-
-from zope.testing import doctest
+import doctest
 from zeam.form.base.testing import FunctionalLayer
 
+
 def test_suite():
     optionflags = doctest.NORMALIZE_WHITESPACE | doctest.ELLIPSIS
-    globs= {}
+    globs = {}
 
     suite = unittest.TestSuite()
     for filename in ['components.txt', 'actions.txt', 'fields.txt',
