@@ -1,4 +1,4 @@
-# -*- coding: utf-8 -*-
+import warnings
 
 from grokcore import component as grok
 from zeam.form.base import interfaces
@@ -6,7 +6,7 @@ from zeam.form.base.components import Component, Collection
 from zeam.form.base.interfaces import IModeMarker, IWidgetFactory
 from zeam.form.base.markers import NO_VALUE, getValue, HiddenMarker
 from zope import component
-from zope.interface import Interface
+from zope.interface import Interface, implementer
 from zope.pagetemplate.interfaces import IPageTemplate
 
 
@@ -19,10 +19,17 @@ def widgetId(form, component):
          if iditem))
 
 
-class WidgetFactory(object):
+def getWidgetExtractor(field, form, request):
+    warnings.warn(
+        u"getWidgetExtractor is deprecated in favor of "
+        u"form.widgetFactory.extractor", DeprecationWarning)
+    return form.widgetFactory.extractor(field)
+
+
+@implementer(IWidgetFactory)
+class WidgetFactory:
     """Generic API to create widgets and extractors.
     """
-    grok.implements(IWidgetFactory)
 
     def __init__(self, form, request):
         self.form = form
@@ -60,8 +67,8 @@ class WidgetFactory(object):
             interfaces.IWidgetExtractor)
 
 
+@implementer(interfaces.IWidgets)
 class Widgets(Collection):
-    grok.implements(interfaces.IWidgets)
 
     type = interfaces.IWidget
 
@@ -96,17 +103,24 @@ class Widgets(Collection):
             widget.update()
 
 
+@implementer(interfaces.IWidget)
 class Widget(Component, grok.MultiAdapter):
     grok.baseclass()
-    grok.implements(interfaces.IWidget)
     grok.provides(interfaces.IWidget)
 
+    defaultHtmlAttributes = set(['required', 'readonly', 'placeholder',
+                                 'autocomplete', 'size', 'maxlength',
+                                 'pattern', 'style'])
+    defaultHtmlClass = ['field']
+    alternateLayout = False
+
     def __init__(self, component, form, request):
         identifier = widgetId(form, component)
         super(Widget, self).__init__(component.title, identifier)
         self.component = component
         self.form = form
         self.request = request
+        self._htmlAttributes = {}
 
     def clone(self, new_identifier=None):
         raise NotImplementedError
@@ -116,7 +130,30 @@ class Widget(Component, grok.MultiAdapter):
         return self.identifier.replace('.', '-')
 
     def htmlClass(self):
-        return 'field'
+        result = self.defaultHtmlClass
+        if self.required:
+            result = result + ['field-required',]
+        return ' '.join(result)
+
+    def htmlAttribute(self, name=None):
+        value = self._htmlAttributes.get(name)
+        if value:
+            # Boolean return as value the name of the property
+            if isinstance(value, bool):
+                return name
+            return value
+        return None
+
+    def htmlAttributes(self):
+        attributes = {}
+        for key, value in sorted(self._htmlAttributes.items()):
+            if (value and
+                (key.startswith('data-') or key in self.defaultHtmlAttributes)):
+                if isinstance(value, bool):
+                    attributes[key] = key
+                else:
+                    attributes[key] = str(value)
+        return attributes
 
     def isVisible(self):
         return not isinstance(self.component.mode, HiddenMarker)
@@ -143,8 +180,8 @@ class Widget(Component, grok.MultiAdapter):
         return template()
 
 
+@implementer(interfaces.IWidgetExtractor)
 class WidgetExtractor(grok.MultiAdapter):
-    grok.implements(interfaces.IWidgetExtractor)
     grok.provides(interfaces.IWidgetExtractor)
     grok.adapts(
         interfaces.IRenderableComponent,
@@ -158,20 +195,35 @@ class WidgetExtractor(grok.MultiAdapter):
         self.request = request
 
     def extract(self):
-        value = self.request.form.get(self.identifier, u'')
-        if not len(value):
+        value = self.request.form.get(self.identifier)
+        if value is None:
             value = NO_VALUE
         return (value, None)
 
     def extractRaw(self):
         entries = {}
         sub_identifier = self.identifier + '.'
-        for key, value in self.request.form.iteritems():
+        for key, value in self.request.form.items():
             if key.startswith(sub_identifier) or key == self.identifier:
                 entries[key] = value
         return entries
 
 
+class FieldWidgetExtractor(WidgetExtractor):
+    grok.adapts(
+        interfaces.IField,
+        interfaces.IFieldExtractionValueSetting,
+        Interface)
+
+    def extract(self):
+        value = self.request.form.get(self.identifier)
+        # The value is empty only if the field is required.
+        if (value is None or
+            (not len(value) and self.component.isRequired(self.form))):
+            value = NO_VALUE
+        return (value, None)
+
+
 class HiddenWidgetExtractor(WidgetExtractor):
     grok.name('hidden')
 
@@ -189,18 +241,23 @@ class ActionWidget(Widget):
         interfaces.IFieldExtractionValueSetting,
         Interface)
     grok.name('input')
+    defaultHtmlAttributes = set(['accesskey', 'formnovalidate', 'style'])
+    defaultHtmlClass = ['action']
 
     def __init__(self, component, form, request):
         super(ActionWidget, self).__init__(component, form, request)
         self.description = component.description
-        self.accesskey = component.accesskey
+        self._htmlAttributes.update({
+                'accesskey': component.accesskey,
+                'formnovalidate': not component.html5Validation})
+        self._htmlAttributes.update(component.htmlAttributes)
 
     def htmlClass(self):
         return 'action'
 
 
+@implementer(interfaces.IFieldWidget)
 class FieldWidget(Widget):
-    grok.implements(interfaces.IFieldWidget)
     grok.adapts(
         interfaces.IField,
         interfaces.IFormData,
@@ -211,7 +268,10 @@ class FieldWidget(Widget):
         super(FieldWidget, self).__init__(component, form, request)
         self.description = component.description
         self.required = component.isRequired(form)
-        self.readonly = component.readonly
+        self._htmlAttributes.update(component.htmlAttributes)
+        self._htmlAttributes.update({
+                'readonly': component.readonly,
+                'required': self.required})
 
     @property
     def error(self):
@@ -246,7 +306,7 @@ class FieldWidget(Widget):
         return self.prepareContentValue(value)
 
     def valueToUnicode(self, value):
-        return unicode(value)
+        return value
 
     def prepareRequestValue(self, value, extractor):
         return value
